<?php
function query(string $query, array $data = []) {
    try {
        $string = "mysql:host=" . DBHOST . ";dbname=" . DBNAME;
        $con = new PDO($string, DBUSER, DBPASS);
        $con->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        $stm = $con->prepare($query);
        $stm->execute($data);

        // Check if the query is a SELECT query
        if (stripos(trim($query), 'SELECT') === 0) {
            $result = $stm->fetchAll(PDO::FETCH_ASSOC);
            return (!empty($result)) ? $result : false;
        }

        // For INSERT, UPDATE, DELETE, return true if rows were affected
        return ($stm->rowCount() > 0);
        
    } catch (PDOException $e) {
        echo json_encode(["error" => "Database error: " . $e->getMessage()]);
        return false;
    }
}


function query_row(string $query,array $data=[])         // $data can be empty that's why the array was empty
{
    try {
        $string = "mysql:host=" . DBHOST . ";dbname=" . DBNAME;
        $con = new PDO($string, DBUSER, DBPASS);
        //  echo "Connected successfully";
    } catch (PDOException $e) {
        echo "Connection failed: " . $e->getMessage();
    }
//  echo $query;
// echo $data;
    $stm=$con->prepare($query);
    $stm->execute($data);

    $result=$stm->fetchAll(PDO::FETCH_ASSOC);      // :: is used to provide values rather than providing in query  , it prevents sql injection   
    if(is_array($result) && !empty($result)){ 
        return $result[0];
    }
    return false;
} 

function str_to_url($url){
    $url = str_replace("'","",$url);
    $url = preg_replace('~[^\\pL0-9]+~u', '-', $url);
    $url = trim($url, "-");
    $url = iconv("utf-8", "us-ascii//TRANSLIT", $url);
    $url = strtolower($url);
    $url = preg_replace('~[^-a-z0-9_]+~', '', $url);
    return $url;
}

function get_pagination_vars()
{
  /** set pagination vars */

$page_number = $_GET['page'] ?? 1;
$page_number = empty($page_number) ? 1 : (int)$page_number;
$page_number = $page_number < 1 ? 1 : $page_number;

$current_link = $_GET['url'] ?? 'home';
$current_link = ROOT . '/' . $current_link;
$query_string = "";

foreach ($_GET as $key => $value) {
    if ($key != 'url') {
        $query_string .= "&".$key."=".$value;
    }
}

if(!strstr($query_string, 'page=')){
    $query_string .= "&page=". $page_number;
}

$query_string = trim($query_string, "&");
$current_link .= "?" . $query_string;

$current_link = preg_replace("/page=.*/", "page=".$page_number, $current_link);
$next_link = preg_replace("/page=.*/", "page=".($page_number + 1), $current_link);
$first_link = preg_replace("/page=.*/", "page=1", $current_link);
$prev_page_number = $page_number < 2 ? 1 : $page_number - 1;
$prev_link = preg_replace("/page=.*/", "page=".$prev_page_number, $current_link);

// echo $current_link;
// echo $next_link;
// echo $prev_link;
// echo $first_link;

$result = [
        'current_link'  => $current_link,
        'next_link'     => $next_link,
        'prev_link'     => $prev_link,
        'first_link'    => $first_link,
        'page_number'   => $page_number
        
];

return $result;
}

function authenticate(array $row)
{
    $_SESSION['USER']=$row[0];
    $_SESSION['user_id']=$row[0]['id'];
}

function user($key){
    
    if(empty($key)){
        return $_SESSION['USER'];
    }
    if(isset($_SESSION['USER'])){
        return $_SESSION['USER'][$key];
    }
    return '';
}

function logged_in()
{
    return isset($_SESSION['USER']);
}
function redirect(string $page)
{
    header('Location: ' .$page);
    die;
}

function oldvalue($key,$default='')
{
    if(isset($_POST[$key])){
        echo $_POST[$key];
    }
    else{
        echo $default;
    }
}


function old_checked($key,$default='')
{
    if(isset($_POST[$key])){
        echo 'checked';
    }
    return '';
}

function get_image($image)
{
    if($image){
        return $image;
    }
    return '/uploads/default.jpg';
}

function esc(string $string)
{
    return htmlspecialchars($string ?? '');
}
 
//phpinfo();
function find($table, $id = null, $column = '*') {
    $st = "";
    $params = [];

    if (!empty($id)) {
        $st = "id = :id AND ";  // Using `user_id` instead of `id` for flexibility
        $params['id'] = intval($id);
    }

    $query = "SELECT $column FROM $table WHERE {$st}active = 1";
    
    // If counting posts, modify the query accordingly
    if ($column === 'COUNT(id) AS count') {
        $st = "user_id = :id AND ";  // Using `user_id` instead of `id` for flexibility
        $params['id'] = intval($id);
        $query = "SELECT COUNT(id) AS count FROM $table WHERE {$st}active = 1";
    } else {
        $query .= " LIMIT 1";
    }

    $result = query($query, $params);

    if ($column === 'COUNT(id) AS count') {
        return $result[0]['count'] ?? 0; // Return the count value
    }

    return $result ? $result[0][$column] ?? '' : '';
}



function resize_image($filename, $max_size = 1000)
{
    if (!file_exists($filename)) {
       return false;
    }


    $type = mime_content_type($filename);
    switch ($type) {
        case 'image/jpeg':
            $image = imagecreatefromjpeg($filename);
            break;  
        case 'image/png':
            $image = imagecreatefrompng($filename);
            break;
        case 'image/gif':
            $image = imagecreatefromgif($filename);
            break;
        case 'image/webp':
            $image = imagecreatefromwebp($filename);
            break;
        default:
            return false; // Unsupported file type
    }

    if (!$image) {
        return false; // Failed to create image
    }

    // Get original dimensions
    $src_width = imagesx($image);
    $src_height = imagesy($image);

    // Calculate new dimensions
    if ($src_width > $src_height) {
        $new_width = min($max_size, $src_width);
        $new_height = round(($src_height / $src_width) * $new_width);
    } else {
        $new_height = min($max_size, $src_height);
        $new_width = round(($src_width / $src_height) * $new_height);
    }

    // Create a new resized image
    $new_image = imagecreatetruecolor($new_width, $new_height);

    // Preserve transparency for PNG & GIF
    if ($type === 'image/png' || $type === 'image/gif') {
        imagecolortransparent($new_image, imagecolorallocatealpha($new_image, 0, 0, 0, 127));
        imagealphablending($new_image, false);
        imagesavealpha($new_image, true);
    }

    imagecopyresampled($new_image, $image, 0, 0, 0, 0, $new_width, $new_height, $src_width, $src_height);

    // Save the resized image
    switch ($type) {
        case 'image/jpeg':
            imagejpeg($new_image, $filename, 90);
            break;
        case 'image/png':
            imagepng($new_image, $filename, 9);
            break;
        case 'image/gif':
            imagegif($new_image, $filename);
            break;
        case 'image/webp':
            imagewebp($new_image, $filename, 90);
            break;
    }

    imagedestroy($image);
    imagedestroy($new_image);

    return true;
}
function timepicker($date){
    
    $seconds_ago = (time() - strtotime($date));

    if ($seconds_ago >= 31536000) {
        $date =  intval($seconds_ago / 31536000) . " years ago";
    } elseif ($seconds_ago >= 2419200) {
        $date =  intval($seconds_ago / 2419200) . " months ago";
    } elseif ($seconds_ago >= 86400) {
        $date =  intval($seconds_ago / 86400) . " days ago";
    } elseif ($seconds_ago >= 3600) {
        $date =  intval($seconds_ago / 3600) . " hours ago";
    } elseif ($seconds_ago >= 120) {
        $date =  intval($seconds_ago / 60) . " minutes ago";
    } elseif ($seconds_ago >= 60) {
        $date = "a minute ago";
    } else {
        $date = "less than a minute ago";
    }
    return $date;
}
function getFollowCounts($user_id){
    $queryFollowers = "SELECT COUNT(*) AS followers FROM followers WHERE following_id = :id";
    $queryFollowing = "SELECT COUNT(*) AS following FROM followers WHERE follower_id = :id";

    $followers = query($queryFollowers, ['id' => $user_id]);
    $following = query($queryFollowing, ['id' => $user_id]);

    return [
        'followers' => $followers[0]['followers'] ?? 0,
        'following' => $following[0]['following'] ?? 0
    ];
}